# jsutils api
js工具类

[时间示例演示：http://zhtt.gitee.io/angular-demo/angular/dist/angular/#/orders](http://zhtt.gitee.io/angular-demo/angular/dist/angular/#/orders)

[数组示例演示：http://zhtt.gitee.io/angular-demo/angular/dist/angular/#/utils/array-test](http://zhtt.gitee.io/angular-demo/angular/dist/angular/#/utils/array-test)
## 安装
```
npm i great-jsutils --save
```
## License
请查看 [MIT license](./LICENSE).

## 数组
```js
import { ArrayUtils } from "great-jsutils/index";
```
### 添加元素
**ArrayUtils.add(array: any, obj: any, {idKey : "id", pIdKey : "parentId", childKey : "children"} = {});**

| 参数 | 名称 | 必传 | 参数类型
| ------ | ------ | ------ | ------ |
| array | 被操作的数组 | 是 | Array |
| obj | 被添加的元素 | 是 | any |
| keys | 主键、子属性对应关系 | 否 | string |

**示例1：添加元素**
```js
function add01() {
    let data = [{
        "id": "id-A", "type": "1", "children": [
            {
                "id": "id-A01", "type": "2", "children": [
                    {"id": "id-A01-01", "type": "3"}
                ]
            },
            {"id": "id-A03", "type": "3"}
        ]
    }];
    let child = {"id": "id-A03-01", "parentId": "id-A03"};
    let result = ArrayUtils.add(data, child);
    console.log(result,JSON.stringify(data));
}
function add02() {
    let data = [{
        "orgId": "id-A", "type": "1", "children": [
            {"orgId": "id-A01", "type": "2"},
            {"orgId": "id-A03", "type": "3"}
        ]
    }];
    let child = {"orgId": "id-A03-01", "parentId": "id-A03"};
    let result = ArrayUtils.add(data, child,{idKey:"orgId"});
    console.log(result,JSON.stringify(data));
}
function add03() {
    let data = [{
        "orgId": "id-A", "type": "1", "items": [
            {"orgId": "id-A01", "type": "2"},
            {"orgId": "id-A03", "type": "3"}
        ]
    }];
    let child = {"orgId": "id-A03-01", "parentId": "id-A03"};
    let result = ArrayUtils.add(data, child,{idKey:"orgId",childKey:"items"});
    console.log(result,JSON.stringify(data));
}
function add04() {
    let data = [{
        "id": "id-A", "type": "1", "children": [
            {"id": "id-A03", "type": "3"}
        ]
    }];
    let keys = {"id": "key"};
    let child = {"id": "id-A03-01", "parent": {"id": "id-A03"}};
    let result = ArrayUtils.add(data, child,{"pIdKey":"parent"});
    console.log(result,JSON.stringify(data));
}
```
### 移除元素
**ArrayUtils.remove(array,value,key);**

| 参数 | 名称 | 必传 | 参数类型
| ------ | ------ | ------ | ------ |
| array | 被操作的数组 | 是 | Array |
| value | 需要移除的值 | 是 | any |
| keys | 主键、子属性对应关系 | 否 | string |

**示例1：移除字符串**
```js
//字符串移除，第一个参数是被操作的数组，第二个参数是需要移除的字符串
testArray(){
    let arr=["a","b","c"];
    ArrayUtils.remove(arr,"b");
    console.log(arr.join(","));//a,c
}
```
**示例2：对象移除**
```js
function removeRoot01() {
    let objArr=[{"id":"1","name":"赵云"},{"id":"2","name":"诸葛亮"}];
    ArrayUtils.remove(objArr,{"id":"2"});
    console.log(JSON.stringify(objArr));//[{"id":"1","name":"赵云"}]
}
```
**示例3：对象移除**
```js
function removeRoot02() {
    let objArr=[{"userId":"1","name":"赵云"},{"userId":"2","name":"诸葛亮"}];
    ArrayUtils.remove(objArr,{"userId":"2"},{idKey:"userId"});
    console.log(JSON.stringify(objArr));//[{"userId":"1","name":"赵云"}]
}
```
**示例4：子节点移除**
```js
function removeChild01() {
    let objArr=[{"id":"1","name":"山东省","children":[{"id":"2","name":"济南市"}]}];
    ArrayUtils.remove(objArr,{"id":"2"});
    console.log(JSON.stringify(objArr));//[{"id":"1","name":"赵云"}]
}
function removeChild02() {
    let objArr=[{"id":"1","name":"山东省","items":[{"id":"2","name":"济南市"}]}];
    ArrayUtils.remove(objArr,{"id":"2"},{childKey:"items"});
    console.log(JSON.stringify(objArr));//[{"id":"1","name":"赵云"}]
}
```
### 更新元素
**ArrayUtils.update(array,obj,keys);**

| 参数 | 名称 | 必传 | 参数类型
| ------ | ------ | ------ | ------ |
| array | 被操作的数组 | 是 | Array |
| obj | 需要更新的对象 | 是 | any |
| keys | 主键、子属性对应关系 | 否 | string |

**示例1：更新根节点中的元素**
```js
//字符串移除，第一个参数是被操作的数组，第二个参数是需要更新的对象

function updateRoot01() {
    let obj={"id":"001","name":"001"};
    let array=[{"id":"001","name":"111"}];
    let newArray=ArrayUtils.update(array,obj);
    console.log(JSON.stringify(array));
}
```

**示例2：更新根节点中的元素**
```js
import { ArrayUtils } from "great-jsutils/index";

function updateRoot02() {
    let obj={"userId":"002","name":"222"};
    let array=[{"userId":"002","name":"111"}];
    let newArray=ArrayUtils.update(array,obj,{idKey:"userId"});
    console.log(JSON.stringify(array));
}
```
**示例3：更新子节点中的元素**
```js
import { ArrayUtils } from "great-jsutils/index";
function updateChildren01() {
    let obj={"id":"001","name":"001"};
    let array=[{"id":"a","name":"111","children":[{"id":"001","name":"222"}]}];
    ArrayUtils.update(array,obj);
    console.log(JSON.stringify(array));
}
```
**示例4：更新子节点中的元素**
```js
import { ArrayUtils } from "great-jsutils/index";
function updateChildren02() {
    let obj={"userId":"001","name":"001"};
    let array=[{"userId":"A","name":"A","items":[{"userId":"001","name":"222"}]}];
    ArrayUtils.update(array,obj,{idKey:"userId",childKey:"items"});
    console.log(JSON.stringify(array));
}
```

**示例5：更新时，增加key
```javascript

function updateChildren03() {
    let obj={"userId":"001","name":"001","checked":true};
    let array=[{"userId":"A","name":"A","items":[{"userId":"001","name":"222"}]}];
    ArrayUtils.update(array,obj,{idKey:"userId",childKey:"items"},function(oldData,newData){
        return {"checked":newData["checked"]};
    });
    console.log(JSON.stringify(array));

}
```
### 合并元素
**ArrayUtils.merge(array,obj,keys);**

| 参数 | 名称 | 必传 | 参数类型
| ------ | ------ | ------ | ------ |
| array | 被操作的数组 | 是 | Array |
| obj | 需要更新的对象 | 是 | any |
| keys | 主键、子属性对应关系 | 否 | string |
> 与update类似，这个会增加第二个参数中的新属性

### 属性替换
**ArrayUtils.updateKey(array,keys,function);**

| 参数 | 名称 | 必传 | 参数类型
| ------ | ------ | ------ | ------ |
| array | 被操作的数组 | 是 | Array |
| keys | 原key与新key的对应关系 | 是 | json |
| function | 自定义需要添加的属性 | 否 | string |

**示例1：属性替换**
```js
import { ArrayUtils } from "great-jsutils/index";
//数组中的对象key替换，例如源数据为[{"name":"张三"}]替换为[{"text":"张三"}]
testArray(){
    let datas=[{"name":"张三","sex":"1"}];
    let keys={"name":"text"};
    let result=ArrayUtils.updateKey(datas,keys);
    console.log(result);//{name: "张三", sex: "1"}
}
```

**示例2：属性替换，并添加新属性**
```js
//数组中对象key替换，例如源数据为[{"name":"张三","age":80}]替换为[{"text":"张三","age":80,"ageGroup":"老年人"}];
//这里是根据某些值特征，增加新的属性
testArray(){
    let datas=[{"name":"张三","sex":"1"}];
    let keys={"name":"text"};
    let result=ArrayUtils.updateKey(data,keys,function(){
        if(data["age"]>75){
            return {"ageGroup":"老年人"};
        }
    });
    console.log(result);//[{"text":"张三","age":80,"ageGroup":"老年人"}];
}
```

### 获取树节点
**ArrayUtils.get(array,obj,keys);**

| 参数 | 名称 | 必传 | 参数类型
| ------ | ------ | ------ | ------ |
| array | 被操作的数组 | 是 | Array |
| obj | 需要更新的对象 | 是 | any |
| keys | 主键、子属性对应关系 | 否 | string |

**示例1：根据主键获取**
```js
//从树状数组中，根据id的值获取节点数据

function get01() {
    let data = [ {"id": "id-A01", "type": "2"},{"id": "id-A03", "type": "3"}];
    let item = ArrayUtils.get(data, "id-A03");
    console.log(item);//{"id": "id-A03", "type": "3"}
}
function get02() {
    let data = [ {"userId": "id-A01", "type": "2"},{"userId": "id-A03", "type": "3"}];
    let item = ArrayUtils.get(data, "id-A03",{idKey:"userId"});
    console.log(item);//{"id": "id-A03", "type": "3"}
}
function get03() {
    let data = [ {"userId": "id-A01", "type": "2"},{"userId": "id-A03", "type": "3"}];
    let item = ArrayUtils.get(data, {"id":"id-A03"});
    console.log(item);//{"id": "id-A03", "type": "3"}
}
function get() {
    let data = [{
        "id": "id-A", "type": "1", "children": [
            {
                "id": "id-A01", "type": "2", "children": [
                    {
                        "id": "id-A01-01", "type": "3", "children": [
                            {"id": "id-A01-01-01", "type": "3-01", "children": [
                                    {"id": "id-A01-01-01", "type": "3-02"}
                                ]}
                        ]
                    }, {
                        "id": "id-A01-02", "type": "3", "children": [
                            {"id": "id-A01-02-01", "type": "3-01", "children": [
                                    {"id": "id-A01-02-01-01", "type": "3-02", "children": [
                                            {"id": "id-A01-02-01-01-01", "type": "3-02"}
                                        ]}
                                ]}
                        ]
                    }
                ]
            },
            {"id": "id-A03", "type": "3"}
        ]
    }];
    let item = ArrayUtils.get(data, "id-A01-02-01-01-01");
    console.log(item);//{ id: 'id-A01-02-01-01-01', type: '3-02' }
}
```
**示例2：根据主键获取**
```js
//从树状数组中，根据id的值获取节点数据
getTreeNodeById(){
    let arr=[{"id":"id-A","type":"1","children":[
                {"id":"id-A01","type":"2","children":[
                    {"id":"id-A01-01","type":"3","children":[
                        {"id":"id-A01-01-01","type":"3-01"}
                    ]},
                    {"id":"id-A01-02","type":"3","children":[
                        {"id":"id-A01-02-01","type":"3-02"}
                    ]}
                ]},
                {"id":"id-A03","type":"3"}
            ]}];
    let item=ArrayUtils.get(arr,"id-A01-02-01","id","children");
    console.log(item);//{ id: 'id-A01-02-01', type: '3-02' }
}
```
### 是否是个有效的数组对象
**ArrayUtils.valid(array);**
>是否是个有效的数组对象，有则返回第一个子元素，否则返回false

| 参数 | 名称 | 必传 | 参数类型
| ------ | ------ | ------ | ------ |
| array | 被操作的数组 | 是 | Array |

**示例1：是否是个有效的数组对象**
```js
//从树状数组中，根据id的值获取节点数据
valid(){
    let arr=[{"id":"id-A","type":"1","children":[
                {"id":"id-A01","type":"2","children":[
                    {"id":"id-A01-01","type":"3","children":[
                        {"id":"id-A01-01-01","type":"3-01"}
                    ]},
                    {"id":"id-A01-02","type":"3","children":[
                        {"id":"id-A01-02-01","type":"3-02"}
                    ]}
                ]},
                {"id":"id-A03","type":"3"}
            ]}];
    let item=ArrayUtils.valid(arr);
    console.log(item);
}
```
### 节点移动操作
```javascript

/**
* 将第二个元素下移2个位置
*/
function move01() {
   let arr=["a","b","c","d","e"];
    //ArrayUtils
    let index=2;
    let size=2;
    ArrayUtils.down(arr,index,{size});
    console.log(arr);
}
/**
* 将第二个元素下移到第2个位置
*/
function move01() {
   let arr=["a","b","c","d","e"];
    //ArrayUtils
    let index=2;
    let end=2;
    ArrayUtils.down(arr,index,{end});
    console.log(arr);
}
/**
* 将第二个元素移动到底部
*/
function moveBottom() {
    let arr = ["a", "b", "c", "d", "e"];
    //ArrayUtils
    let index = 2;
    ArrayUtils.bottom(arr,index);
    console.log(arr);
}
```
## 加密
**示例1：md5**
```js
//字符串的加密
import {JsUtilsApi } from "great-jsutils/index";
function md5() {
  JsUtilsApi.md5("admin123456")
}
```
**示例2：Base64**
```js
//字符串的加密、解密
import {Base64 } from "great-jsutils/index";
function base64() {
console.log('base64：',Base64.encode("123"));
console.log('base64：',Base64.decode("MTIz"));
}
```

## 时间工具类
```js
import { TimeUtils } from "great-jsutils/index";
```
**判断是否是日期类型**
```js
TimeUtils.isDate(args);
```
**获取某天开始、结束时间**
**TimeUtils.getDayTime(n:number);**

| 参数 | 名称 | 必传 | 参数类型
| ------ | ------ | ------ | ------ |
| n | 获取n天前时间，不传则为当天时间 | 否 | number |

>根据传入的数字，获取n天前的开始结束时间，正数则往后推，负数则往前推
```js
import {TimeUtils } from "great-jsutils/index";
function testTime(){
    console.log("当天开始、结束时间",TimeUtils.getDayTime());
    //当天开始、结束时间 { start: '2018-08-31 00:00:00', end: '2018-08-31 24:59:59' }
    console.log("获取昨天时间：",TimeUtils.getDayTime(-1));
    console.log("获取前天时间：",TimeUtils.getDayTime(-2));
    console.log("获取明天时间：",TimeUtils.getDayTime(1));
    console.log("获取后天时间：",TimeUtils.getDayTime(1));
}
```
**获取某周开始、结束时间**
```js 
TimeUtils.getWeekTime(n:number); 
```
根据传入的数字，获取n周前的开始结束时间，正数则往后推，负数则往前推
```js
import {TimeUtils } from "great-jsutils/index";
function testTime(){
    console.log("本周开始、结束时间",TimeUtils.getWeekTime());
    //本周开始、结束时间 { start: '2018-09-17 00:00:00', end: '2018-09-23 24:59:59' }
    console.log("上周开始、结束时间",TimeUtils.getWeekTime(-1));
    //上周开始、结束时间 { start: '2018-09-10 00:00:00', end: '2018-09-16 24:59:59' }
}
```
**获取某月开始、结束时间**
```js
 TimeUtils.getMonthTime(n:number);
 ```
根据传入的数字，获取n月前的开始结束时间，正数则往后推，负数则往前推
```js
import {TimeUtils } from "great-jsutils/index";
function testTime(){
    console.log("本月开始、结束时间",TimeUtils.getMonthTime());
    //本月开始、结束时间 { start: '2018-09-01 00:00:00', end: '2018-09-30 24:59:59' }
    console.log("上月开始、结束时间",TimeUtils.getMonthTime(-1));
    //上月开始、结束时间 { start: '2018-08-01 00:00:00', end: '2018-08-31 24:59:59' }
}
```
**获取某季度开始、结束时间**
```js 
TimeUtils.getSeasonTime(n:number); 
```
根据传入的数字，获取n月前的开始结束时间，正数则往后推，负数则往前推
```js
import {TimeUtils } from "great-jsutils/index";
function testTime(){
    console.log("本季度开始结束时间",TimeUtils.getQuarterTime());
    //本季度开始结束时间 { start: '2018-07-01 00:00:00', end: '2018-09-30 24:59:59' }
    console.log("上季度开始结束时间",TimeUtils.getQuarterTime(-1));
}
```
**获取某年开始、结束时间**
```js 
TimeUtils.getYearTime(n:number); 
```
根据传入的数字，获取n年前的开始结束时间，正数则往后推，负数则往前推
```js
import {TimeUtils } from "great-jsutils/index";
function testTime(){
    console.log("本年开始结束时间",TimeUtils.getYearTime());
    //本年开始结束时间 { start: '2018-01-01 00:00:00', end: '2018-12-31 24:59:59' }
    console.log("上年开始结束时间",TimeUtils.getYearTime(-1));
}
```
**获取N天内的开始、结束时间**
>getWinthInDayTime

| 参数 | 名称 | 必传 | 参数类型
| ------ | ------ | ------ | ------ |
| num | 指定天数 | 否 | number |
```
function getWinthInDayTime(){
    let num1 = -29;// 向前推获取30天内的时间
    let time1 = TimeUtils.getWinthInDayTime(num1);
    console.log(`\n${num1}天内的开始、结束时间：\n`, time1);
    /*
     -29天内的开始、结束时间：
     { start: '2019-03-23 00:00:00', end: '2019-04-21 24:59:59' }
     */

    let num2 = 29; // 向后推，获取30天内的时间
    let time2 = TimeUtils.getWinthInDayTime(num2);
    console.log(`\n${num2}天内的开始、结束时间：\n`, time2);
    /*
     29天内的开始、结束时间：
     { start: '2019-04-21 00:00:00', end: '2019-05-20 24:59:59' }
     */
}
```

**格式化时间**
**TimeUtils.format(date:Date,format:string)**

| 参数 | 名称 | 必传 | 参数类型
| ------ | ------ | ------ | ------ |
| date | 需要格式化的时间 | 否 | Date |
| format | 需要转换的格式 | 否 | string |
```js
import {TimeUtils } from "great-jsutils/index";
function testTime(){
    console.log("格式化时间（默认的格式化类型）：",TimeUtils.format());//当前时间
    //格式化时间（默认的格式化类型）： 2018-09-23 17:18:19
    console.log("格式化时间（默认的格式化类型）：",TimeUtils.format(new Date()));
    //格式化时间（默认的格式化类型）： 2018-09-23 17:18:19
    console.log("格式化时间（根据传入的参数格式）：",TimeUtils.format(new Date(),"yyyy-MM-dd hh时"));
    //格式化时间（根据传入的参数格式）： 2018-09-23 17时
    console.log("格式化时间（API提供的类型）：",TimeUtils.format(new Date(),TimeUtils.FORMAT_YMD));
    //格式化时间（API提供的类型）： 2018-09-23
}
```
**生成一个随机时间**
```javascript
/* 获取一个随机时间 */
function random() {
    let date =  TimeUtils.random();
    let dateStr = TimeUtils.format(date);
    console.log(dateStr);
}
random();
```
**根据当前时间，输出早上好、上午好……**
```javascript
function sayHello() {
    let hello =  TimeUtils.sayHello();
    console.log(hello);
}
sayHello();
```
**字符串转时间**
```js
function parse(){
    let date = TimeUtils.parse("2019年11年9日 11点11分11秒");
    if(date){
        console.log('parse：',TimeUtils.format(date));
        //parse： 2019-11-09 11:11:11
    }else{
        console.log('null');
    }
}
```
## 字符串
```js
import { StringUtils } from "great-jsutils/index";
```
**示例1：删除全部空格**
```js
import {StringUtils } from "great-jsutils/index";
StringUtils.trim(str);
```
**示例2：删除前后空格**
```js
import {StringUtils } from "great-jsutils/index";
StringUtils.trimAround(str);
```

## json
```js
import { JsonUtils } from "great-jsutils/index";
```
**示例1：判断是否为json**
```js
import {JsonUtils } from "great-jsutils/index";
function json(){
    let str1="123";
    console.log(str1,JsonUtils.isJsonString(str1));
    let str2="{'name':'1'}";
    console.log(str2,JsonUtils.isJsonString(str2));
    let str3="{name:'1'}";
    console.log(str3,JsonUtils.isJsonString(str3));
    let str4='{"name":"xt"}';
    console.log(str4,JsonUtils.isJsonString(str4));
}
```
**示例2：字符串转json**
```js
import {JsonUtils } from "great-jsutils/index";
let str4='{"name":"xt"}';
JsonUtils.toJson(str4);
```
**示例3：syntaxHighlight**
```js
import {JsonUtils } from "great-jsutils/index";
let str4='{"name":"xt"}';
JsonUtils.syntaxHighlight(str4);
```

## 正则
```js
import {JsUtilsApi, ArrayUtils, Base64, TimeUtils } from "great-jsutils/index";

```

## 对象操作

```js
import {ObjectUtils} from "great-jsutils/index";

```


```js
/**
 * 清除对象中的空值
 */
function obj01(){
    let obj={a:null,b:"",c:2,d:"aa"};
    ObjectUtils.filter(obj);
    console.log('obj\t',obj); //obj   { c: 2, d: 'aa' }
}

```


```js
/**
 * 清除对象中的空值。
 * 并指定其他需要强制移除的key
 */
function obj02(){
    let obj={a:null,b:"",c:2,d:"aa"};
    let removeKeys=["d"];
    ObjectUtils.filter(obj,{removeKeys});
    console.log('obj\t',obj);//obj   { c: 2}
}

```


```js
/**
 * 根据指定函数确定需要移除的key
 * 下面示例将删除值长度为2 或 值为b的key
 */
function obj03(){
    let obj={a:"dd",b:"b",c:"ddd",d:"aa"};
    function fun(v){
        return v=="b" || v.length==2;
    }
    ObjectUtils.filter(obj,{fun});
    console.log('obj\t',obj);// { c: 'ddd' }
}

```

## number
### 1.1随机生成一组整数
>默认返回一个小于10的整数，可以指定返回的最大值、最小值，以及返回的个数
```js
import { NumberUtils } from "great-jsutils/index";
```
**示例1：随机生成一个小于10的整数**
```js
NumberUtils.randomInt();
```
**示例2：随机生成一个小于100的整数**
```js
NumberUtils.randomInt({max:100});
```
**示例3：随机生成四个大于等于10，小于100的整数**
```js
NumberUtils.randomInt({max:100,min:10,number:4})
```

### 1.2随机生成一组浮点数
>默认返回一个小于10的浮点数，可以指定返回的最大值、最小值、小数位数，以及返回的个数

**示例1：随机生成一个小于10的两位小数的浮点数**
```js
NumberUtils.randomInt();
```
**示例2：随机生成一个小于100的两位小数的浮点数**
```js
NumberUtils.randomInt({max:100});
```
**示例3：随机生成四个大于等于10，小于100的两位小数的浮点数**
```js
NumberUtils.randomInt({max:100,min:10,number:4})
```
**示例4：随机生成四个大于等于10，小于100的三位小数的浮点数**
```js
NumberUtils.randomInt({max:100,min:10,number:4,scale3})
```
## 日志LogUtils 

## SessionStroage
